import { BrowserPlugin } from "@snowplow/browser-tracker-core";
import { CommonEventProperties, DynamicContext } from "@snowplow/tracker-core";
/** Type of media player event */
declare enum MediaEventType$0 {
    // Controlling the playback
    /** Media player event fired when the media tracking is successfully attached to the player and can track events. */
    Ready = "ready",
    /** Media player event sent when the player changes state to playing from previously being paused. */
    Play = "play",
    /** Media player event sent when the user pauses the playback. */
    Pause = "pause",
    /** Media player event sent when playback stops when end of the media is reached or because no further data is available. */
    End = "end",
    /** Media player event sent when a seek operation begins. */
    SeekStart = "seek_start",
    /** Media player event sent when a seek operation completes. */
    SeekEnd = "seek_end",
    // Changes in playback settings
    /** Media player event sent when the playback rate has changed. */
    PlaybackRateChange = "playback_rate_change",
    /** Media player event sent when the volume has changed. */
    VolumeChange = "volume_change",
    /** Media player event fired immediately after the browser switches into or out of full-screen mode. */
    FullscreenChange = "fullscreen_change",
    /** Media player event fired immediately after the browser switches into or out of picture-in-picture mode. */
    PictureInPictureChange = "picture_in_picture_change",
    // Tracking playback progress
    /** Media player event fired periodically during main content playback, regardless of other API events that have been sent. */
    Ping = "ping",
    /** Media player event fired when a percentage boundary set in options.boundaries is reached */
    PercentProgress = "percent_progress",
    // Ad events
    /** Media player event that signals the start of an ad break. */
    AdBreakStart = "ad_break_start",
    /** Media player event that signals the end of an ad break. */
    AdBreakEnd = "ad_break_end",
    /** Media player event that signals the start of an ad. */
    AdStart = "ad_start",
    /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
    AdFirstQuartile = "ad_first_quartile",
    /** Media player event fired when a midpoint of ad is reached after continuous ad playback at normal speed. */
    AdMidpoint = "ad_midpoint",
    /** Media player event fired when a quartile of ad is reached after continuous ad playback at normal speed. */
    AdThirdQuartile = "ad_third_quartile",
    /** Media player event that signals the ad creative was played to the end at normal speed. */
    AdComplete = "ad_complete",
    /** Media player event fired when the user activated a skip control to skip the ad creative. */
    AdSkip = "ad_skip",
    /** Media player event fired when the user clicked on the ad. */
    AdClick = "ad_click",
    /** Media player event fired when the user clicked the pause control and stopped the ad creative. */
    AdPause = "ad_pause",
    /** Media player event fired when the user resumed playing the ad creative after it had been stopped or paused. */
    AdResume = "ad_resume",
    // Data quality events
    /** Media player event fired when the player goes into the buffering state and begins to buffer content. */
    BufferStart = "buffer_start",
    /** Media player event fired when the the player finishes buffering content and resumes playback. */
    BufferEnd = "buffer_end",
    /** Media player event tracked when the video playback quality changes automatically. */
    QualityChange = "quality_change",
    /** Media player event tracked when the resource could not be loaded due to an error.  */
    Error = "error"
}
/**
 * Configuration for filtering out repeated events of the same type tracked after each other.
 * By default, the seek start, seek end and volume change events are filtered out.
 */
type FilterOutRepeatedEvents = {
    /**
     * Whether to filter out seek start and end events tracked after each other.
     */
    seekEvents?: boolean;
    /**
     * Whether to filter out volume change events tracked after each other.
     */
    volumeChangeEvents?: boolean;
    /**
     * Timeout in milliseconds after which to send the events that are queued for filtering.
     * Defaults to 5000 ms.
     */
    flushTimeoutMs?: number;
} | boolean;
type MediaTrackingConfiguration = {
    /** Unique ID of the media tracking. The same ID will be used for media player session if enabled. */
    id: string;
    /** Attributes for the media player context entity */
    player?: MediaPlayerUpdate;
    /** Attributes for the media player session context entity or false to disable it. Enabled by default. */
    session?: {
        /** Local date-time timestamp of when the session started. Automatically set to current time if not given. */
        startedAt?: Date;
    } | false;
    /** Configuration for sending ping events. Enabled by default.  */
    pings?: {
        /** Interval in seconds for sending ping events. Defaults to 30s. */
        pingInterval?: number;
        /** Maximum number of consecutive ping events to send when playback is paused. Defaults to 1. */
        maxPausedPings?: number;
    } | boolean;
    /** Update page activity in order to keep sending page ping events while playing. Enabled by default. */
    updatePageActivityWhilePlaying?: boolean;
    /** Percentage boundaries when to send percent progress events. Disabled by default. */
    boundaries?: number[];
    /**
     * List of event types to allow tracking.
     * If not specified, all tracked events will be allowed and tracked.
     * Otherwise, tracked event types not present in the list will be discarded.
     */
    captureEvents?: MediaEventType$0[];
    /**
     * Whether to filter out repeated events of the same type tracked after each other.
     * Useful to filter out repeated seek and volume change events tracked when the user holds down the seek or volume control.
     * Only applies to seek and volume change events.
     * Defaults to true.
     */
    filterOutRepeatedEvents?: FilterOutRepeatedEvents;
};
/** Type of media content. */
declare enum MediaType {
    /** Audio content. */
    Audio = "audio",
    /** Video content. */
    Video = "video"
}
/** Partial type/schema for a context entity for media player events with information about the current state of the media player */
interface MediaPlayerUpdate {
    /** The current playback time position within the media in seconds */
    currentTime?: number;
    /** A double-precision floating-point value indicating the duration of the media in seconds */
    duration?: number | null;
    /** If playback of the media has ended */
    ended?: boolean;
    /** Whether the video element is fullscreen */
    fullscreen?: boolean;
    /** Whether the media is a live stream */
    livestream?: boolean;
    /** Human readable name given to tracked media content. */
    label?: string;
    /** If the video should restart after ending */
    loop?: boolean;
    /** Type of media content. */
    mediaType?: MediaType;
    /** If the media element is muted */
    muted?: boolean;
    /** If the media element is paused */
    paused?: boolean;
    /** Whether the video element is showing picture-in-picture */
    pictureInPicture?: boolean;
    /** Type of the media player (e.g., com.youtube-youtube, com.vimeo-vimeo, org.whatwg-media_element) */
    playerType?: string;
    /** Playback rate (1 is normal) */
    playbackRate?: number;
    /** Quality level of the playback (e.g., 1080p). */
    quality?: string;
    /** Volume level */
    volume?: number;
}
interface CommonMediaEventProperties extends Omit<CommonEventProperties, "context"> {
    /** Add context entities to an event by setting an Array of Self Describing JSON */
    context?: DynamicContext;
}
type Config = {
    video: string | HTMLMediaElement;
    label?: string;
} & CommonMediaEventProperties & Omit<MediaTrackingConfiguration, "player">;
declare function MediaTrackingPlugin(): BrowserPlugin;
/**
 * Enables media tracking on an HTML `<video>` or `<audio>` element.
 *
 * @param {Config} config - Configuration options for media tracking.
 *
 * **Required:**
 * - `config.id` (`string`): The session ID. Must be unique for each media element. Used to identify the media session and end tracking.
 * - `config.video` (string | HTMLMediaElement): The ID of the media element or the element itself.
 *
 * **Optional:**
 * - `config.label` (`string`): A human-readable label for the media element.
 * - `config.captureEvents` {@link HTML5MediaEventTypes}: A list of media events to track. All events are tracked by default.
 * - `config.boundaries` (`number[]`): Percentage thresholds (0-100) at which to trigger progress events. Disabled by default.
 * - `config.context` {@link DynamicContext}: Contexts to attach to each tracking event.
 *
 * **Ping Configuration (Optional):**
 * - `config.pings.pingInterval` (`number`): The interval (in seconds) for sending ping events. Default is 30(s).
 * - `config.pings.maxPausedPings` (`number`): The maximum number of ping events sent while playback is paused. Default is 1.
 *
 * **Other Options (Optional):**
 * - `config.updatePageActivityWhilePlaying` (`boolean`): Whether to update page activity while media is playing. Enabled by default.
 * - `config.filterOutRepeatedEvents` {@link FilterOutRepeatedEvents}: Whether to suppress consecutive identical events. Default is false.
 */
declare function startHtml5MediaTracking(config: Config): void;
/**
 * Ends media tracking with the given ID if previously started. Clears local state for the media tracking and sends any events waiting to be sent.
 *
 * @param {string} id - The session ID.
 */
declare function endHtml5MediaTracking(id: string): void;
export { MediaTrackingPlugin, startHtml5MediaTracking, endHtml5MediaTracking };
